"use client";
import * as React from "react";
import Avatar from "@mui/material/Avatar";
import Button from "@mui/material/Button";
import CssBaseline from "@mui/material/CssBaseline";
import TextField from "@mui/material/TextField";
import FormControlLabel from "@mui/material/FormControlLabel";
import Checkbox from "@mui/material/Checkbox";
import Link from "@mui/material/Link";
import Grid from "@mui/material/Grid";
import Box from "@mui/material/Box";
import LockOutlinedIcon from "@mui/icons-material/LockOutlined";
import Typography from "@mui/material/Typography";
import Container from "@mui/material/Container";
import { createTheme, ThemeProvider } from "@mui/material/styles";
import Copyright from "../src/Copyright";
import { useState } from "react";
import validator from "validator";
import axios from "../api/axios";
import { useAuth } from "../contexts/AuthContext";
import { useRouter } from "next/navigation";
import NotRequireLogin from "../guards/NotRequireLogin";

// TODO remove, this demo shouldn't need to reset the theme.
const defaultTheme = createTheme();

export interface FormError {
  field: string;
  error: string;
}
export interface FieldError {
  valid: boolean;
  text: string;
}

export default function SignIn() {

 

  const { login, token  } = useAuth();

  const router = useRouter();
  const [errors, setErrors] = useState<Array<FormError>>([]);

  const handleSubmit = async (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    setErrors([]);
    const data = new FormData(event.currentTarget);
    let newErrors: Array<FormError> = [];

    if (validator.isEmpty(data.get("identifier")!.toString())) {
      newErrors.push({ field: "identifier", error: "Required field" });
    }
    if (validator.isEmpty(data.get("password")!.toString())) {
      newErrors.push( { field: "password", error: "Required field" })
    }
    setErrors(newErrors);

    if (newErrors.length === 0) {
      
    //   console.log({
    //     identifier: data.get("identifier"),
    //     password: data.get("password"),
    //     remember: data.get("remember"),
    //   });
      try{
        const response = await axios.post('/auth/login',JSON.stringify({
            username: data.get('identifier')?.toString(),
            password: data.get('password')?.toString(),
            remember: data.get('remember') == null? false: true,
        }),{
            headers: {
                'Content-Type': 'application/json',
              },
        })
        const { access_token, exp } = response.data;
        login(access_token, data.get('remember') == null? false: true, exp);
        router.push('/dashboard');
      }catch(e){
        setErrors((prevErrors) => [
            ...prevErrors,
            { field: "identifier", error: "Invalid username or password" },
          ]);
      }
    }
  };

  const getFieldError = (field: string): FieldError => {
    const find = errors.find((item) => item.field == field);
    if (find == undefined) {
      return {
        valid: false,
        text: "",
      };
    } 
    return {
        valid: true,
        text: find.error,
      };
  };

  return (
    <NotRequireLogin>
    <ThemeProvider theme={defaultTheme}>
      <Container component="main" maxWidth="xs">
        <CssBaseline />
        <Box
          sx={{
            marginTop: 8,
            display: "flex",
            flexDirection: "column",
            alignItems: "center",
          }}
        >
          <Avatar sx={{ m: 1, bgcolor: "secondary.main" }}>
            <LockOutlinedIcon />
          </Avatar>
          <Typography component="h1" variant="h5">
            Sign in
          </Typography>
          <Box
            component="form"
            onSubmit={handleSubmit}
            noValidate
            sx={{ mt: 1 }}
          >
            <TextField
              error={getFieldError("identifier").valid}
              margin="normal"
              required
              fullWidth
              id="identifier"
              label="Username Or Email"
              name="identifier"
              autoComplete="identifier"
              autoFocus
              helperText={getFieldError("identifier").text}
            />
            <TextField
             error={getFieldError("password").valid}
              margin="normal"
              required
              fullWidth
              name="password"
              label="Password"
              type="password"
              id="password"
              autoComplete="current-password"
              helperText={getFieldError("password").text}
            />
            <FormControlLabel
              control={
                <Checkbox value="remember" name="remember" color="primary" />
              }
              label="Remember me"
            />
            <Button
              type="submit"
              fullWidth
              variant="contained"
              sx={{ mt: 3, mb: 2 }}
            >
              Sign In
            </Button>
            {/* <Grid container>
              <Grid item xs>
                <Link href="#" variant="body2">
                  Forgot password?
                </Link>
              </Grid>
              <Grid item>
                <Link href="#" variant="body2">
                  {"Don't have an account? Sign Up"}
                </Link>
              </Grid>
            </Grid> */}
          </Box>
        </Box>
        <Copyright sx={{ mt: 8, mb: 4 }} />
      </Container>
    </ThemeProvider>
    </NotRequireLogin>
  );
}
